@@ -25,10 +25,7 @@ use std::io::{BufWriter, Write};
 
 use flatbuffers::FlatBufferBuilder;
 
-use crate::array::{
-    as_large_list_array, as_list_array, as_map_array, as_struct_array, as_union_array,
-    make_array, Array, ArrayData, ArrayRef, FixedSizeListArray,
-};
+use crate::array::{ArrayData, ArrayRef};
 use crate::buffer::{Buffer, MutableBuffer};
 use crate::datatypes::*;
 use crate::error::{ArrowError, Result};
@@ -39,7 +36,7 @@ use crate::util::bit_util;
 use ipc::CONTINUATION_MARKER;
 
 /// IPC write options used to control the behaviour of the writer
-#[derive(Debug, Clone)]
+#[derive(Debug)]
 pub struct IpcWriteOptions {
     /// Write padding after memory buffers to this multiple of bytes.
     /// Generally 8 or 64, defaults to 8
@@ -140,134 +137,26 @@ impl IpcDataGenerator {
         }
     }
 
-    fn _encode_dictionaries(
+    pub fn encoded_batch(
         &self,
-        column: &ArrayRef,
-        encoded_dictionaries: &mut Vec<EncodedData>,
+        batch: &RecordBatch,
         dictionary_tracker: &mut DictionaryTracker,
         write_options: &IpcWriteOptions,
-    ) -> Result<()> {
-        match column.data_type() {
-            DataType::Struct(fields) => {
-                let s = as_struct_array(column);
-                for (field, &column) in fields.iter().zip(s.columns().iter()) {
-                    self.encode_dictionaries(
-                        field,
-                        column,
-                        encoded_dictionaries,
-                        dictionary_tracker,
-                        write_options,
-                    )?;
-                }
-            }
-            DataType::List(field) => {
-                let list = as_list_array(column);
-                self.encode_dictionaries(
-                    field,
-                    &list.values(),
-                    encoded_dictionaries,
-                    dictionary_tracker,
-                    write_options,
-                )?;
-            }
-            DataType::LargeList(field) => {
-                let list = as_large_list_array(column);
-                self.encode_dictionaries(
-                    field,
-                    &list.values(),
-                    encoded_dictionaries,
-                    dictionary_tracker,
-                    write_options,
-                )?;
-            }
-            DataType::FixedSizeList(field, _) => {
-                let list = column
-                    .as_any()
-                    .downcast_ref::<FixedSizeListArray>()
-                    .expect("Unable to downcast to fixed size list array");
-                self.encode_dictionaries(
-                    field,
-                    &list.values(),
-                    encoded_dictionaries,
-                    dictionary_tracker,
-                    write_options,
-                )?;
-            }
-            DataType::Map(field, _) => {
-                let map_array = as_map_array(column);
+    ) -> Result<(Vec<EncodedData>, EncodedData)> {
+        // TODO: handle nested dictionaries
+        let schema = batch.schema();
+        let mut encoded_dictionaries = Vec::with_capacity(schema.fields().len());
 
-                let (keys, values) = match field.data_type() {
-                    DataType::Struct(fields) if fields.len() == 2 => {
-                        (&fields[0], &fields[1])
-                    }
-                    _ => panic!("Incorrect field data type {:?}", field.data_type()),
-                };
+        for (i, field) in schema.fields().iter().enumerate() {
+            let column = batch.column(i);
 
-                // keys
-                self.encode_dictionaries(
-                    keys,
-                    &map_array.keys(),
-                    encoded_dictionaries,
-                    dictionary_tracker,
-                    write_options,
-                )?;
-
-                // values
-                self.encode_dictionaries(
-                    values,
-                    &map_array.values(),
-                    encoded_dictionaries,
-                    dictionary_tracker,
-                    write_options,
-                )?;
-            }
-            DataType::Union(fields, _) => {
-                let union = as_union_array(column);
-                for (field, ref column) in fields
-                    .iter()
-                    .enumerate()
-                    .map(|(n, f)| (f, union.child(n as i8)))
-                {
-                    self.encode_dictionaries(
-                        field,
-                        column,
-                        encoded_dictionaries,
-                        dictionary_tracker,
-                        write_options,
-                    )?;
-                }
-            }
-            _ => (),
-        }
-
-        Ok(())
-    }
-
-    fn encode_dictionaries(
-        &self,
-        field: &Field,
-        column: &ArrayRef,
-        encoded_dictionaries: &mut Vec<EncodedData>,
-        dictionary_tracker: &mut DictionaryTracker,
-        write_options: &IpcWriteOptions,
-    ) -> Result<()> {
-        match column.data_type() {
-            DataType::Dictionary(_key_type, _value_type) => {
+            if let DataType::Dictionary(_key_type, _value_type) = column.data_type() {
                 let dict_id = field
                     .dict_id()
                     .expect("All Dictionary types have `dict_id`");
                 let dict_data = column.data();
                 let dict_values = &dict_data.child_data()[0];
 
-                let values = make_array(dict_data.child_data()[0].clone());
-
-                self._encode_dictionaries(
-                    &values,
-                    encoded_dictionaries,
-                    dictionary_tracker,
-                    write_options,
-                )?;
-
                 let emit = dictionary_tracker.insert(dict_id, column)?;
 
                 if emit {
@@ -278,38 +167,10 @@ impl IpcDataGenerator {
                     ));
                 }
             }
-            _ => self._encode_dictionaries(
-                column,
-                encoded_dictionaries,
-                dictionary_tracker,
-                write_options,
-            )?,
-        }
-
-        Ok(())
-    }
-
-    pub fn encoded_batch(
-        &self,
-        batch: &RecordBatch,
-        dictionary_tracker: &mut DictionaryTracker,
-        write_options: &IpcWriteOptions,
-    ) -> Result<(Vec<EncodedData>, EncodedData)> {
-        let schema = batch.schema();
-        let mut encoded_dictionaries = Vec::with_capacity(schema.all_fields().len());
-
-        for (i, field) in schema.fields().iter().enumerate() {
-            let column = batch.column(i);
-            self.encode_dictionaries(
-                field,
-                column,
-                &mut encoded_dictionaries,
-                dictionary_tracker,
-                write_options,
-            )?;
         }
 
         let encoded_message = self.record_batch_to_bytes(batch, write_options);
+
         Ok((encoded_dictionaries, encoded_message))
     }
 
@@ -606,18 +467,6 @@ impl<W: Write> FileWriter<W> {
 
         Ok(())
     }
-
-    /// Unwraps the BufWriter housed in FileWriter.writer, returning the underlying
-    /// writer
-    ///
-    /// The buffer is flushed and the FileWriter is finished before returning the
-    /// writer.
-    pub fn into_inner(mut self) -> Result<W> {
-        if !self.finished {
-            self.finish()?;
-        }
-        self.writer.into_inner().map_err(ArrowError::from)
-    }
 }
 
 pub struct StreamWriter<W: Write> {
@@ -625,6 +474,8 @@ pub struct StreamWriter<W: Write> {
     writer: BufWriter<W>,
     /// IPC write options
     write_options: IpcWriteOptions,
+    /// A reference to the schema, used in validating record batches
+    schema: Schema,
     /// Whether the writer footer has been written, and the writer is finished
     finished: bool,
     /// Keeps track of dictionaries that have been written
@@ -653,6 +504,7 @@ impl<W: Write> StreamWriter<W> {
         Ok(Self {
             writer,
             write_options,
+            schema: schema.clone(),
             finished: false,
             dictionary_tracker: DictionaryTracker::new(false),
             data_gen,
@@ -852,9 +704,9 @@ fn write_continuation<W: Write>(
 /// Write array data to a vector of bytes
 fn write_array_data(
     array_data: &ArrayData,
-    buffers: &mut Vec<ipc::Buffer>,
-    arrow_data: &mut Vec<u8>,
-    nodes: &mut Vec<ipc::FieldNode>,
+    mut buffers: &mut Vec<ipc::Buffer>,
+    mut arrow_data: &mut Vec<u8>,
+    mut nodes: &mut Vec<ipc::FieldNode>,
     offset: i64,
     num_rows: usize,
     null_count: usize,
@@ -862,11 +714,7 @@ fn write_array_data(
     let mut offset = offset;
     nodes.push(ipc::FieldNode::new(num_rows as i64, null_count as i64));
     // NullArray does not have any buffers, thus the null buffer is not generated
-    // UnionArray does not have a validity buffer
-    if !matches!(
-        array_data.data_type(),
-        DataType::Null | DataType::Union(_, _)
-    ) {
+    if array_data.data_type() != &DataType::Null {
         // write null buffer if exists
         let null_buffer = match array_data.null_buffer() {
             None => {
@@ -879,11 +727,11 @@ fn write_array_data(
             Some(buffer) => buffer.clone(),
         };
 
-        offset = write_buffer(&null_buffer, buffers, arrow_data, offset);
+        offset = write_buffer(&null_buffer, &mut buffers, &mut arrow_data, offset);
     }
 
     array_data.buffers().iter().for_each(|buffer| {
-        offset = write_buffer(buffer, buffers, arrow_data, offset);
+        offset = write_buffer(buffer, &mut buffers, &mut arrow_data, offset);
     });
 
     if !matches!(array_data.data_type(), DataType::Dictionary(_, _)) {
@@ -892,9 +740,9 @@ fn write_array_data(
             // write the nested data (e.g list data)
             offset = write_array_data(
                 data_ref,
-                buffers,
-                arrow_data,
-                nodes,
+                &mut buffers,
+                &mut arrow_data,
+                &mut nodes,
                 offset,
                 data_ref.len(),
                 data_ref.null_count(),
@@ -976,7 +824,7 @@ mod tests {
             let file =
                 File::open(format!("target/debug/testdata/{}.arrow_file", "arrow"))
                     .unwrap();
-            let mut reader = FileReader::try_new(file, None).unwrap();
+            let mut reader = FileReader::try_new(file).unwrap();
             while let Some(Ok(read_batch)) = reader.next() {
                 read_batch
                     .columns()
@@ -1024,7 +872,7 @@ mod tests {
 
         {
             let file = File::open(&file_name).unwrap();
-            let reader = FileReader::try_new(file, None).unwrap();
+            let reader = FileReader::try_new(file).unwrap();
             reader.for_each(|maybe_batch| {
                 maybe_batch
                     .unwrap()
@@ -1080,7 +928,6 @@ mod tests {
             "generated_interval",
             "generated_datetime",
             "generated_dictionary",
-            "generated_map",
             "generated_nested",
             "generated_primitive_no_batches",
             "generated_primitive_zerolength",
@@ -1094,7 +941,7 @@ mod tests {
             ))
             .unwrap();
 
-            let mut reader = FileReader::try_new(file, None).unwrap();
+            let mut reader = FileReader::try_new(file).unwrap();
 
             // read and rewrite the file to a temp location
             {
@@ -1115,7 +962,7 @@ mod tests {
                 version, path
             ))
             .unwrap();
-            let mut reader = FileReader::try_new(file, None).unwrap();
+            let mut reader = FileReader::try_new(file).unwrap();
 
             // read expected JSON output
             let arrow_json = read_gzip_json(version, path);
@@ -1132,7 +979,6 @@ mod tests {
             "generated_interval",
             "generated_datetime",
             "generated_dictionary",
-            "generated_map",
             "generated_nested",
             "generated_primitive_no_batches",
             "generated_primitive_zerolength",
@@ -1146,7 +992,7 @@ mod tests {
             ))
             .unwrap();
 
-            let reader = StreamReader::try_new(file, None).unwrap();
+            let reader = StreamReader::try_new(file).unwrap();
 
             // read and rewrite the stream to a temp location
             {
@@ -1165,7 +1011,7 @@ mod tests {
             let file =
                 File::open(format!("target/debug/testdata/{}-{}.stream", version, path))
                     .unwrap();
-            let mut reader = StreamReader::try_new(file, None).unwrap();
+            let mut reader = StreamReader::try_new(file).unwrap();
 
             // read expected JSON output
             let arrow_json = read_gzip_json(version, path);
@@ -1185,7 +1031,6 @@ mod tests {
             "generated_dictionary",
             // "generated_duplicate_fieldnames",
             "generated_interval",
-            "generated_map",
             "generated_nested",
             // "generated_nested_large_offsets",
             "generated_null_trivial",
@@ -1203,7 +1048,7 @@ mod tests {
             ))
             .unwrap();
 
-            let mut reader = FileReader::try_new(file, None).unwrap();
+            let mut reader = FileReader::try_new(file).unwrap();
 
             // read and rewrite the file to a temp location
             {
@@ -1229,7 +1074,7 @@ mod tests {
                 version, path
             ))
             .unwrap();
-            let mut reader = FileReader::try_new(file, None).unwrap();
+            let mut reader = FileReader::try_new(file).unwrap();
 
             // read expected JSON output
             let arrow_json = read_gzip_json(version, path);
@@ -1249,7 +1094,6 @@ mod tests {
             "generated_dictionary",
             // "generated_duplicate_fieldnames",
             "generated_interval",
-            "generated_map",
             "generated_nested",
             // "generated_nested_large_offsets",
             "generated_null_trivial",
@@ -1267,7 +1111,7 @@ mod tests {
             ))
             .unwrap();
 
-            let reader = StreamReader::try_new(file, None).unwrap();
+            let reader = StreamReader::try_new(file).unwrap();
 
             // read and rewrite the stream to a temp location
             {
@@ -1290,7 +1134,7 @@ mod tests {
             let file =
                 File::open(format!("target/debug/testdata/{}-{}.stream", version, path))
                     .unwrap();
-            let mut reader = StreamReader::try_new(file, None).unwrap();
+            let mut reader = StreamReader::try_new(file).unwrap();
 
             // read expected JSON output
             let arrow_json = read_gzip_json(version, path);
@@ -1313,67 +1157,4 @@ mod tests {
         let arrow_json: ArrowJson = serde_json::from_str(&s).unwrap();
         arrow_json
     }
-
-    #[test]
-    fn track_union_nested_dict() {
-        let inner: DictionaryArray<Int32Type> = vec!["a", "b", "a"].into_iter().collect();
-
-        let array = Arc::new(inner) as ArrayRef;
-
-        // Dict field with id 2
-        let dctfield =
-            Field::new_dict("dict", array.data_type().clone(), false, 2, false);
-
-        let types = Buffer::from_slice_ref(&[0_i8, 0, 0]);
-        let offsets = Buffer::from_slice_ref(&[0_i32, 1, 2]);
-
-        let union =
-            UnionArray::try_new(types, Some(offsets), vec![(dctfield, array)]).unwrap();
-
-        let schema = Arc::new(Schema::new(vec![Field::new(
-            "union",
-            union.data_type().clone(),
-            false,
-        )]));
-
-        let batch = RecordBatch::try_new(schema, vec![Arc::new(union)]).unwrap();
-
-        let gen = IpcDataGenerator {};
-        let mut dict_tracker = DictionaryTracker::new(false);
-        gen.encoded_batch(&batch, &mut dict_tracker, &Default::default())
-            .unwrap();
-
-        // Dictionary with id 2 should have been written to the dict tracker
-        assert!(dict_tracker.written.contains_key(&2));
-    }
-
-    #[test]
-    fn track_struct_nested_dict() {
-        let inner: DictionaryArray<Int32Type> = vec!["a", "b", "a"].into_iter().collect();
-
-        let array = Arc::new(inner) as ArrayRef;
-
-        // Dict field with id 2
-        let dctfield =
-            Field::new_dict("dict", array.data_type().clone(), false, 2, false);
-
-        let s = StructArray::from(vec![(dctfield, array)]);
-        let struct_array = Arc::new(s) as ArrayRef;
-
-        let schema = Arc::new(Schema::new(vec![Field::new(
-            "struct",
-            struct_array.data_type().clone(),
-            false,
-        )]));
-
-        let batch = RecordBatch::try_new(schema, vec![struct_array]).unwrap();
-
-        let gen = IpcDataGenerator {};
-        let mut dict_tracker = DictionaryTracker::new(false);
-        gen.encoded_batch(&batch, &mut dict_tracker, &Default::default())
-            .unwrap();
-
-        // Dictionary with id 2 should have been written to the dict tracker
-        assert!(dict_tracker.written.contains_key(&2));
-    }
 }
